@@ -17,7 +17,7 @@ use Text::Abbrev();
 use Exporter ();
 our @ISA = qw{ Exporter };
 
-our $VERSION = '0.001';
+our $VERSION = '0.002';
 
 our @EXPORT_OK = qw{
     __am_or_pm
@@ -26,29 +26,46 @@ our @EXPORT_OK = qw{
     __day_of_week
     __format
     __is_leap_year
-    __holiday_name __holiday_name_to_number __holiday_abbr
+    __holiday_abbr __holiday_name __holiday_narrow
+    __holiday_name_to_number
     __month_name __month_name_to_number __month_abbr
     __on_date __on_date_accented
     __quarter __quarter_name __quarter_abbr
     __rata_die_to_year_day
-    __trad_weekday_name __trad_weekday_abbr
+    __trad_weekday_abbr __trad_weekday_name __trad_weekday_narrow
     __valid_date_class
-    __weekday_name __weekday_abbr
+    __weekday_abbr __weekday_name __weekday_narrow
     __week_of_year
     __year_day_to_rata_die
+    DAY_OF_YEAR_MIDYEARS_DAY
+    DAY_OF_YEAR_OVERLITHE
     GREGORIAN_RATA_DIE_TO_SHIRE
+    HOLIDAY_2_YULE
+    HOLIDAY_1_LITHE
+    HOLIDAY_MIDYEARS_DAY
+    HOLIDAY_OVERLITHE
+    HOLIDAY_2_LITHE
+    HOLIDAY_1_YULE
 };
 our %EXPORT_TAGS = (
-    all	=> \@EXPORT_OK,
+    all		=> \@EXPORT_OK,
+    subs	=> [ grep { m/ \A __ /smx } @EXPORT_OK ],
+    consts	=> [ grep { m/ \A [[:upper:]] /smx } @EXPORT_OK ],
 );
 
 use constant ARRAY_REF	=> ref [];
 use constant CODE_REF	=> ref sub {};
 use constant HASH_REF	=> ref {};
 
-use constant MIDYEAR_DAY_OF_YEAR	=> 183;
-use constant OVERLITHE_DAY_NUMBER	=> 4;
-use constant OVERLITHE_DAY_OF_YEAR	=> 184;
+use constant DAY_OF_YEAR_MIDYEARS_DAY	=> 183;
+use constant DAY_OF_YEAR_OVERLITHE	=> 184;
+
+use constant HOLIDAY_2_YULE		=> 1;
+use constant HOLIDAY_1_LITHE		=> 2;
+use constant HOLIDAY_MIDYEARS_DAY	=> 3;
+use constant HOLIDAY_OVERLITHE		=> 4;
+use constant HOLIDAY_2_LITHE		=> 5;
+use constant HOLIDAY_1_YULE		=> 6;
 
 # See the documentation below for where the value came from.
 
@@ -81,7 +98,7 @@ use constant GREGORIAN_RATA_DIE_TO_SHIRE	=> 1995694;
 }
 
 {
-    my @holiday_day = ( undef, 1, 182, 183, OVERLITHE_DAY_OF_YEAR, 185, 366 );
+    my @holiday_day = ( undef, 1, 182, 183, DAY_OF_YEAR_OVERLITHE, 185, 366 );
     my @month_zero = ( undef, 1, 31, 61, 91, 121, 151, 185, 215, 245,
 	275, 305, 335 );
 
@@ -95,9 +112,9 @@ use constant GREGORIAN_RATA_DIE_TO_SHIRE	=> 1995694;
 
 	unless ( __is_leap_year( $year ) ) {
 	    not $month
-		and OVERLITHE_DAY_NUMBER == $day
+		and HOLIDAY_OVERLITHE == $day
 		and Carp::croak( 'Overlithe only occurs in a leap year' );
-	    $yd >= OVERLITHE_DAY_OF_YEAR
+	    $yd >= DAY_OF_YEAR_OVERLITHE
 		and --$yd;
 	}
 	return $yd;
@@ -109,7 +126,7 @@ use constant GREGORIAN_RATA_DIE_TO_SHIRE	=> 1995694;
 	my ( $year, $yd ) = $validate_doy2d->( @_ );
 
 	unless ( __is_leap_year( $year ) ) {
-	    $yd >= OVERLITHE_DAY_OF_YEAR
+	    $yd >= DAY_OF_YEAR_OVERLITHE
 		and $yd++;
 	}
 	$yd > 0
@@ -207,12 +224,18 @@ sub _fmt_cond {
 		    return _fmt_number_02( @_[ 0, 1 ],
 			$_[0]->__fmt_shire_day() );
 		},
+	Ea	=> sub { $_[0]->__fmt_shire_traditional() ?
+		    __trad_weekday_narrow( $_[0]->__fmt_shire_day_of_week() ) :
+		    __weekday_narrow( $_[0]->__fmt_shire_day_of_week() );
+		},
 	Ed	=> \&_fmt_on_date,
 	EE	=> sub { __holiday_name( $_[0]->__fmt_shire_month() ? 0 :
 		$_[0]->__fmt_shire_day() ) },
 	Ee	=> sub { __holiday_abbr( $_[0]->__fmt_shire_month() ? 0 :
 		$_[0]->__fmt_shire_day() ) },
 	En	=> sub { $_[1]{prefix_new_line_unless_empty}++; '' },
+	Eo	=> sub { __holiday_narrow( $_[0]->__fmt_shire_month() ? 0 :
+		$_[0]->__fmt_shire_day() ) },
 	Ex	=> sub { __format( $_[0],
 		    '%{{%A %-e %B %Y||%A %EE %Y||%EE %Y}}' ) },
 	e	=> sub {
@@ -415,35 +438,35 @@ sub _fmt_on_date {
     return "$pfx$on_date";
 }
 
+{
+    my @name = ( '',
+	'2Yu', '1Li', 'Myd', 'Oli', '2Li', '1Yu',
+    );
+
+    sub __holiday_abbr {
+	return _lookup( $_[0], \@name );
+    }
+}
+
 {
     my @name = ( '',
 	'2 Yule', '1 Lithe', q<Midyear's day>, 'Overlithe', '2 Lithe',
 	'1 Yule',
     );
 
-    my $validate_name = _make_validator( qw{ UInt|Undef } );
-
     sub __holiday_name {
-	my ( $holiday ) = $validate_name->( @_ );
-	defined $holiday
-	    or return [ @name ];
-	return $name[ $holiday || 0 ];
+	return _lookup( $_[0], \@name );
     }
 
 }
 
 {
     my @name = ( '',
-	'2Yu', '1Li', 'Myd', 'Oli', '2Li', '1Yu',
+	'2Y', '1L', 'My', 'Ol', '2L', '1Y',
     );
 
-    my $validate = _make_validator( qw{ UInt|Undef } );
-
-    sub __holiday_abbr {
-	my ( $holiday ) = $validate->( @_ );
-	defined $holiday
-	    or return [ @name ];
-	return $name[ $holiday || 0 ];
+    sub __holiday_narrow {
+	return _lookup( $_[0], \@name );
     }
 }
 
@@ -785,22 +808,24 @@ sub _fmt_on_date {
     my @name = ( '', 'Sterrendei', 'Sunnendei', 'Monendei',
 	'Trewesdei', 'Hevenesdei', 'Meresdei', 'Highdei' );
 
-    my $validate = _make_validator( qw{ UInt } );
-
     sub __trad_weekday_name {
-	my ( $weekday ) = $validate->( @_ );
-	return $name[ $weekday ];
+	return _lookup( $_[0], \@name );
     }
 }
 
 {
     my @name = ( '', 'Ste', 'Sun', 'Mon', 'Tre', 'Hev', 'Mer', 'Hig' );
 
-    my $validate = _make_validator( qw{ UInt } );
-
     sub __trad_weekday_abbr {
-	my ( $weekday ) = $validate->( @_ );
-	return $name[ $weekday ];
+	return _lookup( $_[0], \@name );
+    }
+}
+
+{
+    my @name = ( '', 'St', 'Su', 'Mo', 'Tr', 'He', 'Me', 'Hi' );
+
+    sub __trad_weekday_narrow {
+	return _lookup( $_[0], \@name );
     }
 }
 
@@ -824,22 +849,24 @@ sub _fmt_on_date {
     my @name = ( '', 'Sterday', 'Sunday', 'Monday', 'Trewsday',
 	'Hevensday', 'Mersday', 'Highday' );
 
-    my $validate = _make_validator( qw{ UInt } );
-
     sub __weekday_name {
-	my ( $weekday ) = $validate->( @_ );
-	return $name[ $weekday ];
+	return _lookup( $_[0], \@name );
     }
 }
 
 {
     my @name = ( '', 'Ste', 'Sun', 'Mon', 'Tre', 'Hev', 'Mer', 'Hig' );
 
-    my $validate = _make_validator( qw{ UInt } );
-
     sub __weekday_abbr {
-	my ( $weekday ) = $validate->( @_ );
-	return $name[ $weekday ];
+	return _lookup( $_[0], \@name );
+    }
+}
+
+{
+    my @name = ( '', 'St', 'Su', 'Mo', 'Tr', 'He', 'Me', 'Hi' );
+
+    sub __weekday_narrow {
+	return _lookup( $_[0], \@name );
     }
 }
 
@@ -984,6 +1011,10 @@ sub _make_lookup_hash {
 	# validators in-line, above, and %type_def needs to be populated
 	# before that happens.
 	%type_def = (
+	    # An array reference
+	    Array	=> sub {
+		ARRAY_REF eq ref $_ ? 0 : 'an ARRAY reference'
+	    },
 	    # A hash reference
 	    Hash	=> sub { HASH_REF eq ref $_ ? 0 : 'a HASH reference' },
 	    # An integer, optionally signed
@@ -1091,6 +1122,23 @@ sub _normalize_for_lookup {
     }
 }
 
+{
+    my $validate;
+
+    BEGIN {
+	$validate = _make_validator( qw{ UInt|Undef Array } );
+    }
+
+    sub _lookup {
+	my ( $inx, $tbl ) = $validate->( @_ );
+	defined $inx
+	    and return $tbl->[ $inx ];
+	__PACKAGE__ eq caller
+	    or Carp::croak( 'Index not defined' );
+	return $tbl;
+    }
+}
+
 1;
 
 __END__
@@ -1140,6 +1188,20 @@ as follows:
 
 =back
 
+This module is subroutine-based. Nothing is exported by default, but
+everything public can be exported by name. In addition the following
+export tags exist:
+
+=over
+
+=item C<:all> - export everything;
+
+=item C<:consts> - export all manifest constants;
+
+=item C<:subs> - export all subroutines.
+
+=back
+
 =head1 SUBROUTINES
 
 This class supports the following public subroutines. Anything
@@ -1150,7 +1212,7 @@ concerned, they will be package-private for the purposes of any code
 that uses this package.
 
 All of the following are exportable to your name space, but none are
-exported by default.
+exported by default. They can all be exported using export tag C<:subs>.
 
 =head2 __am_or_pm
 
@@ -1320,6 +1382,11 @@ The day of the month as a decimal number, zero-filled (range C<01> to
 C<30>). On holidays it is the holiday number, zero-filled (range C<01>
 to C<06>).
 
+=item %Ea
+
+The narrow (2-character) weekday name, or C<''> for holidays that are
+part of no week.
+
 =item %Ed
 
 The L<__on_date()|/__on_date> text for the given date.
@@ -1341,6 +1408,10 @@ The abbreviated holiday name, or C<''> for non-holidays.
 Inserts nothing, but causes the next C<%Ed> (and B<only> the next one)
 to have a C<"\n"> prefixed if there was an actual event on the date.
 
+=item %Eo
+
+The narrow (2-character) holiday name, or C<''> for non-holidays.
+
 =item %Ex
 
 Like C<'%c'>, but without the time of day, and with full names rather
@@ -1591,6 +1662,15 @@ Immediately after the flags (if any) you can specify a field width that
 overrides the default. If you explicitly specify field width, flag
 C<'-'> pads with spaces, even in numeric fields.
 
+=head2 __holiday_abbr
+
+ say __holiday_abbr( 3 );
+
+Given a holiday number C<(1-6)>, this subroutine returns that holiday's
+three-letter abbreviation. If the holiday number is C<0> (i.e. the day
+is not a holiday), an empty string is returned. Otherwise, C<undef> is
+returned.
+
 =head2 __holiday_name
 
  say __holiday_name( 3 );
@@ -1610,12 +1690,12 @@ number of the holiday. Unique abbreviations of names or short names
 digits are returned unmodified. Anything unrecognized causes C<0> to be
 returned.
 
-=head2 __holiday_abbr
+=head2 __holiday_narrow
 
- say __holiday_abbr( 3 );
+ say __holiday_narrow( 3 );
 
 Given a holiday number C<(1-6)>, this subroutine returns that holiday's
-three-letter abbreviation. If the holiday number is C<0> (i.e. the day
+two-letter abbreviation. If the holiday number is C<0> (i.e. the day
 is not a holiday), an empty string is returned. Otherwise, C<undef> is
 returned.
 
@@ -1744,6 +1824,15 @@ This subroutine computes the three-letter abbreviation of a traditional
 is C<0> (i.e.  Midyear's day or the Overlithe) the empty string is
 returned. Otherwise, C<undef> is returned.
 
+=head2 __trad_weekday_narrow
+
+ say 'Day 1 is ', __trad_weekday_narrow( 1 );
+
+This subroutine computes the two-letter abbreviation of a traditional
+(i.e. old-style) weekday given its number (1-7). If the weekday number
+is C<0> (i.e.  Midyear's day or the Overlithe) the empty string is
+returned. Otherwise, C<undef> is returned.
+
 =head2 __valid_date_class
 
  eval {
@@ -1784,6 +1873,15 @@ given its number (1-7). If the weekday number is C<0> (i.e. Midyear's
 day or the Overlithe) the empty string is returned. Otherwise, C<undef>
 is returned.
 
+=head2 __weekday_narrow
+
+ say 'Day 1 is ', __weekday_narrow( 1 );
+
+This subroutine computes the two-letter abbreviation of a weekday
+given its number (1-7). If the weekday number is C<0> (i.e. Midyear's
+day or the Overlithe) the empty string is returned. Otherwise, C<undef>
+is returned.
+
 =head2 __week_of_year
 
  say '25 Rethe is in week ', __week_of_year( 3, 25 );
@@ -1811,7 +1909,19 @@ Rata Die, as in the days since 1 Yule of Shire year 0.
 =head1 MANIFEST CONSTANTS
 
 The following manifest constants are exportable to your name space. None
-is exported by default.
+is exported by default. They can all be exported using export tag
+C<:consts>.
+
+=head2 DAY_OF_YEAR_MIDYEARS_DAY
+
+This manifest constant represents the day number of C<Midyear's day> in
+the year, i.e. C<183>.
+
+=head2 DAY_OF_YEAR_OVERLITHE
+
+This manifest constant represents the day number of C<Overlithe> in the
+year, i.e. C<184>. Be aware that day C<184> is C<Overlithe> only in a
+leap year; otherwise day C<184> is C<2 Lithe>.
 
 =head2 GREGORIAN_RATA_DIE_TO_SHIRE
 
@@ -1838,6 +1948,36 @@ L<DateTime::Fiction::JRRTolkien::Shire|DateTime::Fiction::JRRTolkien::Shire>
 version 0.21. This is after I adopted that module but before I started
 messing with the computational internals.
 
+=head3 HOLIDAY_2_YULE
+
+This manifest constant represents the holiday number of <2 Yule>, i.e.
+C<1>.
+
+=head3 HOLIDAY_1_LITHE
+
+This manifest constant represents the holiday number of <1 Lithe>, i.e.
+C<2>.
+
+=head3 HOLIDAY_MIDYEARS_DAY
+
+This manifest constant represents the holiday number of <Midyear's day>,
+i.e. C<3>.
+
+=head3 HOLIDAY_OVERLITHE
+
+This manifest constant represents the holiday number of <Overlithe>,
+i.e. C<4>.
+
+=head3 HOLIDAY_2_LITHE
+
+This manifest constant represents the holiday number of <2 Lithe>, i.e.
+C<5>.
+
+=head3 HOLIDAY_1_YULE
+
+This manifest constant represents the holiday number of <1 Yule>, i.e.
+C<6>.
+
 =head1 SEE ALSO
 
 L<Date::Tolkien::Shire|Date::Tolkien::Shire>
